<?php

namespace Siropu\AdsManager\Admin\Controller;

use XF\Mvc\ParameterBag;

class GroupDiscount extends AbstractController
{
     public function actionIndex()
     {
          $viewParams = [
               'groupDiscounts' => $this->finder('Siropu\AdsManager:GroupDiscount')->fetch()
          ];

          return $this->view('Siropu\AdsManager:GroupDiscount\List', 'siropu_ads_manager_group_dicount_list', $viewParams);
     }
     public function actionAdd()
     {
          $groupDiscount = $this->em()->create('Siropu\AdsManager:GroupDiscount');
          return $this->groupDiscountAddEdit($groupDiscount);
     }
     public function actionEdit(ParameterBag $params)
     {
          $groupDiscount = $this->assertGroupDiscountExists($params->group_discount_id);
          return $this->groupDiscountAddEdit($groupDiscount);
     }
     public function actionSave(ParameterBag $params)
     {
          $this->assertPostOnly();

		if ($params->group_discount_id)
		{
			$groupDiscount = $this->assertGroupDiscountExists($params->group_discount_id);
		}
		else
		{
			$groupDiscount = $this->em()->create('Siropu\AdsManager:GroupDiscount');
		}

		$this->groupDiscountSaveProcess($groupDiscount)->run();

		return $this->redirect($this->buildLink('ads-manager/group-discounts'));
     }
     public function actionDelete(ParameterBag $params)
     {
          $groupDiscount = $this->assertGroupDiscountExists($params->group_discount_id);
          $plugin  = $this->plugin('XF:Delete');

          return $plugin->actionDelete(
               $groupDiscount,
               $this->buildLink('ads-manager/group-discounts/delete', $groupDiscount),
               $this->buildLink('ads-manager/group-discounts/edit', $groupDiscount),
               $this->buildLink('ads-manager/group-discounts'),
               $groupDiscount->title
          );
     }
     public function actionToggle()
     {
          $plugin = $this->plugin('XF:Toggle');
		return $plugin->actionToggle('Siropu\AdsManager:GroupDiscount', 'enabled');
     }
     protected function groupDiscountAddEdit(\Siropu\AdsManager\Entity\GroupDiscount $groupDiscount)
	{
          $viewParams = [
               'groupDiscount' => $groupDiscount,
               'packages'      => $this->getPackageRepo()->getPackagesForSelect(),
               'userGroups'    => $this->repository('XF:UserGroup')->getUserGroupTitlePairs()
          ];

          return $this->view('Siropu\AdsManager:GroupDiscount\Edit', 'siropu_ads_manager_group_discount_edit', $viewParams);
     }
     protected function groupDiscountSaveProcess(\Siropu\AdsManager\Entity\GroupDiscount $groupDiscount)
	{
          $input = $this->filter([
               'title'        => 'str',
               'user_groups'  => 'array-uint',
               'packages'     => 'array-uint',
               'discount'     => 'array',
               'enabled'      => 'bool'
          ]);

          $form = $this->formAction();
          $form->basicEntitySave($groupDiscount, $input);

		return $form;
     }
     /**
      * @return \Siropu\AdsManager\Entity\GroupDiscount
      */
     protected function assertGroupDiscountExists($id, $with = null)
     {
          return $this->assertRecordExists('Siropu\AdsManager:GroupDiscount', $id, $with, 'siropu_ads_manager_requested_group_discount_not_found');
     }
}
